@@ -116,9 +116,11 @@ static PyObject* ACL_new(PyTypeObject* type, PyObject* args,
 
     acl->acl = acl_init(0);
     if (acl->acl == NULL) {
+        /* LCOV_EXCL_START */
         PyErr_SetFromErrno(PyExc_IOError);
         Py_DECREF(newacl);
         return NULL;
+        /* LCOV_EXCL_STOP */
     }
 #ifdef HAVE_LEVEL2
     acl->entry_id = ACL_FIRST_ENTRY;
@@ -243,15 +245,12 @@ static int ACL_init(PyObject* obj, PyObject* args, PyObject *keywds) {
 static void ACL_dealloc(PyObject* obj) {
     ACL_Object *self = (ACL_Object*) obj;
     PyObject *err_type, *err_value, *err_traceback;
-    int have_error = PyErr_Occurred() ? 1 : 0;
 
-    if (have_error)
-        PyErr_Fetch(&err_type, &err_value, &err_traceback);
+    PyErr_Fetch(&err_type, &err_value, &err_traceback);
     if(self->acl != NULL && acl_free(self->acl) != 0)
-        PyErr_WriteUnraisable(obj);
-    if (have_error)
-        PyErr_Restore(err_type, err_value, err_traceback);
-    PyObject_DEL(self);
+        PyErr_WriteUnraisable(obj);  /* LCOV_EXCL_LINE */
+    PyErr_Restore(err_type, err_value, err_traceback);
+    Py_TYPE(obj)->tp_free(obj);
 }
 
 /* Converts the acl to a text format */
@@ -262,12 +261,16 @@ static PyObject* ACL_str(PyObject *obj) {
 
     text = acl_to_text(self->acl, NULL);
     if(text == NULL) {
+        /* LCOV_EXCL_START */
         return PyErr_SetFromErrno(PyExc_IOError);
+        /* LCOV_EXCL_STOP */
     }
     ret = PyUnicode_FromString(text);
     if(acl_free(text) != 0) {
+        /* LCOV_EXCL_START */
         Py_XDECREF(ret);
         return PyErr_SetFromErrno(PyExc_IOError);
+        /* LCOV_EXCL_STOP */
     }
     return ret;
 }
@@ -317,12 +320,14 @@ static PyObject* ACL_to_any_text(PyObject *obj, PyObject *args,
 
     text = acl_to_any_text(self->acl, arg_prefix, arg_separator, arg_options);
     if(text == NULL) {
-        return PyErr_SetFromErrno(PyExc_IOError);
+        return PyErr_SetFromErrno(PyExc_IOError);  /* LCOV_EXCL_LINE */
     }
     ret = PyBytes_FromString(text);
     if(acl_free(text) != 0) {
+        /* LCOV_EXCL_START */
         Py_XDECREF(ret);
         return PyErr_SetFromErrno(PyExc_IOError);
+        /* LCOV_EXCL_STOP */
     }
     return ret;
 }
@@ -357,7 +362,7 @@ static PyObject* ACL_check(PyObject* obj, PyObject* args) {
     int eindex;
 
     if((result = acl_check(self->acl, &eindex)) == -1)
-        return PyErr_SetFromErrno(PyExc_IOError);
+        return PyErr_SetFromErrno(PyExc_IOError);  /* LCOV_EXCL_LINE */
     if(result == 0) {
         Py_RETURN_FALSE;
     }
@@ -382,7 +387,7 @@ static PyObject* ACL_richcompare(PyObject* o1, PyObject* o2, int op) {
     acl1 = (ACL_Object*)o1;
     acl2 = (ACL_Object*)o2;
     if((n=acl_cmp(acl1->acl, acl2->acl))==-1)
-        return PyErr_SetFromErrno(PyExc_IOError);
+        return PyErr_SetFromErrno(PyExc_IOError);  /* LCOV_EXCL_LINE */
     switch(op) {
     case Py_EQ:
         ret = n == 0 ? Py_True : Py_False;
@@ -415,7 +420,7 @@ static PyObject* ACL_equiv_mode(PyObject* obj, PyObject* args) {
     mode_t mode;
 
     if(acl_equiv_mode(self->acl, &mode) == -1)
-        return PyErr_SetFromErrno(PyExc_IOError);
+        return PyErr_SetFromErrno(PyExc_IOError);  /* LCOV_EXCL_LINE */
     return PyLong_FromLong(mode);
 }
 #endif
@@ -520,15 +525,17 @@ static PyObject* ACL_get_state(PyObject *obj, PyObject* args) {
 
     size = acl_size(self->acl);
     if(size == -1)
-        return PyErr_SetFromErrno(PyExc_IOError);
+        return PyErr_SetFromErrno(PyExc_IOError);  /* LCOV_EXCL_LINE */
 
     if((ret = PyBytes_FromStringAndSize(NULL, size)) == NULL)
         return NULL;
     buf = PyBytes_AsString(ret);
 
     if((nsize = acl_copy_ext(buf, self->acl, size)) == -1) {
+        /* LCOV_EXCL_START */
         Py_DECREF(ret);
         return PyErr_SetFromErrno(PyExc_IOError);
+        /* LCOV_EXCL_STOP */
     }
 
     return ret;
@@ -548,10 +555,13 @@ static PyObject* ACL_set_state(PyObject *obj, PyObject* args) {
     if((ptr = acl_copy_int(buf)) == NULL)
         return PyErr_SetFromErrno(PyExc_IOError);
 
-    /* Free the old acl. Should we ignore errors here? */
     if(self->acl != NULL) {
-        if(acl_free(self->acl) == -1)
-            return PyErr_SetFromErrno(PyExc_IOError);
+        /* Ignore errors in freeing the previous acl. We already
+           allocated the new acl, and the state of the previous one is
+           suspect if freeing failed (in Linux's libacl, deallocating
+           a valid ACL can't actually happen, so this path is
+           unlikely. */
+        acl_free(self->acl); /* LCOV_EXCL_LINE */
     }
 
     self->acl = ptr;
@@ -582,7 +592,7 @@ static PyObject* ACL_iternext(PyObject *obj) {
     nerr = acl_get_entry(self->acl, self->entry_id, &the_entry_t);
     self->entry_id = ACL_NEXT_ENTRY;
     if(nerr == -1)
-        return PyErr_SetFromErrno(PyExc_IOError);
+        return PyErr_SetFromErrno(PyExc_IOError);  /* LCOV_EXCL_LINE */
     else if(nerr == 0) {
         /* Docs says this is not needed */
         /*PyErr_SetObject(PyExc_StopIteration, Py_None);*/
@@ -695,8 +705,10 @@ static PyObject* ACL_append(PyObject *obj, PyObject *args) {
     if(oldentry != NULL) {
         nret = acl_copy_entry(newentry->entry, oldentry->entry);
         if(nret == -1) {
+            /* LCOV_EXCL_START */
             Py_DECREF(newentry);
             return PyErr_SetFromErrno(PyExc_IOError);
+           /* LCOV_EXCL_STOP */
         }
     }
 
@@ -731,13 +743,17 @@ static int get_tag_qualifier(acl_entry_t entry, tag_qual *tq) {
     void *p;
 
     if(acl_get_tag_type(entry, &tq->tag) == -1) {
+        /* LCOV_EXCL_START */
         PyErr_SetFromErrno(PyExc_IOError);
         return -1;
+        /* LCOV_EXCL_STOP */
     }
     if (tq->tag == ACL_USER || tq->tag == ACL_GROUP) {
         if((p = acl_get_qualifier(entry)) == NULL) {
+            /* LCOV_EXCL_START */
             PyErr_SetFromErrno(PyExc_IOError);
             return -1;
+            /* LCOV_EXCL_STOP */
         }
         if (tq->tag == ACL_USER) {
             tq->uid = *(uid_t*)p;
@@ -775,9 +791,11 @@ static PyObject* Entry_new(PyTypeObject* type, PyObject* args,
     entry = (Entry_Object*)newentry;
 
     if(acl_create_entry(&parent->acl, &entry->entry) == -1) {
+        /* LCOV_EXCL_START */
         PyErr_SetFromErrno(PyExc_IOError);
         Py_DECREF(newentry);
         return NULL;
+        /* LCOV_EXCL_STOP */
     }
     Py_INCREF(parent);
     entry->parent_acl = (PyObject*)parent;
@@ -804,17 +822,14 @@ static int Entry_init(PyObject* obj, PyObject* args, PyObject *keywds) {
 static void Entry_dealloc(PyObject* obj) {
     Entry_Object *self = (Entry_Object*) obj;
     PyObject *err_type, *err_value, *err_traceback;
-    int have_error = PyErr_Occurred() ? 1 : 0;
 
-    if (have_error)
-        PyErr_Fetch(&err_type, &err_value, &err_traceback);
+    PyErr_Fetch(&err_type, &err_value, &err_traceback);
     if(self->parent_acl != NULL) {
         Py_DECREF(self->parent_acl);
         self->parent_acl = NULL;
     }
-    if (have_error)
-        PyErr_Restore(err_type, err_value, err_traceback);
-    PyObject_DEL(self);
+    PyErr_Restore(err_type, err_value, err_traceback);
+    Py_TYPE(obj)->tp_free(obj);
 }
 
 /* Converts the entry to a text format */
@@ -856,13 +871,16 @@ static PyObject* Entry_str(PyObject *obj) {
     case ACL_MASK:
         kind = PyUnicode_FromString("the mask");
         break;
-    default:
+    default: /* LCOV_EXCL_START */
         kind = PyUnicode_FromString("UNKNOWN_TAG_TYPE!");
         break;
+        /* LCOV_EXCL_STOP */
     }
     if (kind == NULL) {
+        /* LCOV_EXCL_START */
         Py_DECREF(format);
         return NULL;
+        /* LCOV_EXCL_STOP */
     }
     PyObject *ret = PyUnicode_Concat(format, kind);
     Py_DECREF(format);
@@ -895,8 +913,10 @@ static PyObject* Entry_get_tag_type(PyObject *obj, void* arg) {
     acl_tag_t value;
 
     if(acl_get_tag_type(self->entry, &value) == -1) {
+        /* LCOV_EXCL_START */
         PyErr_SetFromErrno(PyExc_IOError);
         return NULL;
+        /* LCOV_EXCL_STOP */
     }
 
     return PyLong_FromLong(value);
@@ -932,8 +952,10 @@ static int Entry_set_qualifier(PyObject* obj, PyObject* value, void* arg) {
        this ugly dance with two variables and a pointer that will
        point to one of them. */
     if(acl_get_tag_type(self->entry, &tag) == -1) {
+        /* LCOV_EXCL_START */
         PyErr_SetFromErrno(PyExc_IOError);
         return -1;
+        /* LCOV_EXCL_STOP */
     }
     uid = uidgid;
     gid = uidgid;
@@ -962,8 +984,10 @@ static int Entry_set_qualifier(PyObject* obj, PyObject* value, void* arg) {
       return -1;
     }
     if(acl_set_qualifier(self->entry, p) == -1) {
+        /* LCOV_EXCL_START */
         PyErr_SetFromErrno(PyExc_IOError);
         return -1;
+        /* LCOV_EXCL_STOP */
     }
 
     return 0;
@@ -1028,8 +1052,10 @@ static int Entry_set_permset(PyObject* obj, PyObject* value, void* arg) {
     }
     p = (Permset_Object*)value;
     if(acl_set_permset(self->entry, p->permset) == -1) {
+        /* LCOV_EXCL_START */
         PyErr_SetFromErrno(PyExc_IOError);
         return -1;
+        /* LCOV_EXCL_STOP */
     }
     return 0;
 }
@@ -1053,7 +1079,7 @@ static PyObject* Entry_copy(PyObject *obj, PyObject *args) {
         return NULL;
 
     if(acl_copy_entry(self->entry, other->entry) == -1)
-        return PyErr_SetFromErrno(PyExc_IOError);
+        return PyErr_SetFromErrno(PyExc_IOError);  /* LCOV_EXCL_LINE */
 
     Py_RETURN_NONE;
 }
@@ -1112,17 +1138,14 @@ static int Permset_init(PyObject* obj, PyObject* args, PyObject *keywds) {
 static void Permset_dealloc(PyObject* obj) {
     Permset_Object *self = (Permset_Object*) obj;
     PyObject *err_type, *err_value, *err_traceback;
-    int have_error = PyErr_Occurred() ? 1 : 0;
 
-    if (have_error)
-        PyErr_Fetch(&err_type, &err_value, &err_traceback);
+    PyErr_Fetch(&err_type, &err_value, &err_traceback);
     if(self->parent_entry != NULL) {
         Py_DECREF(self->parent_entry);
         self->parent_entry = NULL;
     }
-    if (have_error)
-        PyErr_Restore(err_type, err_value, err_traceback);
-    PyObject_DEL(self);
+    PyErr_Restore(err_type, err_value, err_traceback);
+    Py_TYPE(obj)->tp_free((PyObject *)obj);
 }
 
 /* Permset string representation */
@@ -1145,7 +1168,7 @@ static PyObject* Permset_clear(PyObject* obj, PyObject* args) {
     Permset_Object *self = (Permset_Object*) obj;
 
     if(acl_clear_perms(self->permset) == -1)
-        return PyErr_SetFromErrno(PyExc_IOError);
+        return PyErr_SetFromErrno(PyExc_IOError);  /* LCOV_EXCL_LINE */
 
     Py_RETURN_NONE;
 }
@@ -1176,8 +1199,10 @@ static int Permset_set_right(PyObject* obj, PyObject* value, void* arg) {
     else
         nerr = acl_delete_perm(self->permset, *(acl_perm_t*)arg);
     if(nerr == -1) {
+        /* LCOV_EXCL_START */
         PyErr_SetFromErrno(PyExc_IOError);
         return -1;
+        /* LCOV_EXCL_STOP */
     }
     return 0;
 }
@@ -1204,7 +1229,7 @@ static PyObject* Permset_add(PyObject* obj, PyObject* args) {
         return NULL;
 
     if(acl_add_perm(self->permset, (acl_perm_t) right) == -1)
-        return PyErr_SetFromErrno(PyExc_IOError);
+        return PyErr_SetFromErrno(PyExc_IOError);  /* LCOV_EXCL_LINE */
 
     Py_RETURN_NONE;
 }
@@ -1231,7 +1256,7 @@ static PyObject* Permset_delete(PyObject* obj, PyObject* args) {
         return NULL;
 
     if(acl_delete_perm(self->permset, (acl_perm_t) right) == -1)
-        return PyErr_SetFromErrno(PyExc_IOError);
+        return PyErr_SetFromErrno(PyExc_IOError);  /* LCOV_EXCL_LINE */
 
     Py_RETURN_NONE;
 }
@@ -1259,7 +1284,7 @@ static PyObject* Permset_test(PyObject* obj, PyObject* args) {
 
     ret = get_perm(self->permset, (acl_perm_t) right);
     if(ret == -1)
-        return PyErr_SetFromErrno(PyExc_IOError);
+        return PyErr_SetFromErrno(PyExc_IOError);  /* LCOV_EXCL_LINE */
 
     if(ret) {
         Py_RETURN_TRUE;
@@ -1327,53 +1352,23 @@ static PyMethodDef ACL_methods[] = {
 /* The definition of the ACL Type */
 static PyTypeObject ACL_Type = {
     PyVarObject_HEAD_INIT(NULL, 0)
-    "posix1e.ACL",
-    sizeof(ACL_Object),
-    0,
-    ACL_dealloc,        /* tp_dealloc */
-    0,                  /* tp_print */
-    0,                  /* tp_getattr */
-    0,                  /* tp_setattr */
-    0,                  /* formerly tp_compare, in 3.0 deprecated, in
-                           3.5 tp_as_async */
-    0,                  /* tp_repr */
-    0,                  /* tp_as_number */
-    0,                  /* tp_as_sequence */
-    0,                  /* tp_as_mapping */
-    0,                  /* tp_hash */
-    0,                  /* tp_call */
-    ACL_str,            /* tp_str */
-    0,                  /* tp_getattro */
-    0,                  /* tp_setattro */
-    0,                  /* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT, /* tp_flags */
-    __ACL_Type_doc__,   /* tp_doc */
-    0,                  /* tp_traverse */
-    0,                  /* tp_clear */
+    .tp_name = "posix1e.ACL",
+    .tp_basicsize = sizeof(ACL_Object),
+    .tp_itemsize = 0,
+    .tp_dealloc = ACL_dealloc,
+    .tp_str = ACL_str,
+    .tp_flags = Py_TPFLAGS_DEFAULT,
+    .tp_doc = __ACL_Type_doc__,
 #ifdef HAVE_LINUX
-    ACL_richcompare,    /* tp_richcompare */
-#else
-    0,                  /* tp_richcompare */
+    .tp_richcompare = ACL_richcompare,
 #endif
-    0,                  /* tp_weaklistoffset */
 #ifdef HAVE_LEVEL2
-    ACL_iter,
-    ACL_iternext,
-#else
-    0,                  /* tp_iter */
-    0,                  /* tp_iternext */
+    .tp_iter = ACL_iter,
+    .tp_iternext = ACL_iternext,
 #endif
-    ACL_methods,        /* tp_methods */
-    0,                  /* tp_members */
-    0,                  /* tp_getset */
-    0,                  /* tp_base */
-    0,                  /* tp_dict */
-    0,                  /* tp_descr_get */
-    0,                  /* tp_descr_set */
-    0,                  /* tp_dictoffset */
-    ACL_init,           /* tp_init */
-    0,                  /* tp_alloc */
-    ACL_new,            /* tp_new */
+    .tp_methods = ACL_methods,
+    .tp_init = ACL_init,
+    .tp_new = ACL_new,
 };
 
 #ifdef HAVE_LEVEL2
@@ -1445,43 +1440,17 @@ static char __Entry_Type_doc__[] =
 /* The definition of the Entry Type */
 static PyTypeObject Entry_Type = {
     PyVarObject_HEAD_INIT(NULL, 0)
-    "posix1e.Entry",
-    sizeof(Entry_Object),
-    0,
-    Entry_dealloc,      /* tp_dealloc */
-    0,                  /* tp_print */
-    0,                  /* tp_getattr */
-    0,                  /* tp_setattr */
-    0,                  /* tp_compare */
-    0,                  /* tp_repr */
-    0,                  /* tp_as_number */
-    0,                  /* tp_as_sequence */
-    0,                  /* tp_as_mapping */
-    0,                  /* tp_hash */
-    0,                  /* tp_call */
-    Entry_str,          /* tp_str */
-    0,                  /* tp_getattro */
-    0,                  /* tp_setattro */
-    0,                  /* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT, /* tp_flags */
-    __Entry_Type_doc__, /* tp_doc */
-    0,                  /* tp_traverse */
-    0,                  /* tp_clear */
-    0,                  /* tp_richcompare */
-    0,                  /* tp_weaklistoffset */
-    0,                  /* tp_iter */
-    0,                  /* tp_iternext */
-    Entry_methods,      /* tp_methods */
-    0,                  /* tp_members */
-    Entry_getsets,      /* tp_getset */
-    0,                  /* tp_base */
-    0,                  /* tp_dict */
-    0,                  /* tp_descr_get */
-    0,                  /* tp_descr_set */
-    0,                  /* tp_dictoffset */
-    Entry_init,         /* tp_init */
-    0,                  /* tp_alloc */
-    Entry_new,          /* tp_new */
+    .tp_name = "posix1e.Entry",
+    .tp_basicsize = sizeof(Entry_Object),
+    .tp_itemsize = 0,
+    .tp_dealloc = Entry_dealloc,
+    .tp_str = Entry_str,
+    .tp_flags = Py_TPFLAGS_DEFAULT,
+    .tp_doc = __Entry_Type_doc__,
+    .tp_methods = Entry_methods,
+    .tp_getset = Entry_getsets,
+    .tp_init = Entry_init,
+    .tp_new = Entry_new
 };
 
 /* Permset type methods */
@@ -1552,43 +1521,17 @@ static char __Permset_Type_doc__[] =
 /* The definition of the Permset Type */
 static PyTypeObject Permset_Type = {
     PyVarObject_HEAD_INIT(NULL, 0)
-    "posix1e.Permset",
-    sizeof(Permset_Object),
-    0,
-    Permset_dealloc,    /* tp_dealloc */
-    0,                  /* tp_print */
-    0,                  /* tp_getattr */
-    0,                  /* tp_setattr */
-    0,                  /* tp_compare */
-    0,                  /* tp_repr */
-    0,                  /* tp_as_number */
-    0,                  /* tp_as_sequence */
-    0,                  /* tp_as_mapping */
-    0,                  /* tp_hash */
-    0,                  /* tp_call */
-    Permset_str,        /* tp_str */
-    0,                  /* tp_getattro */
-    0,                  /* tp_setattro */
-    0,                  /* tp_as_buffer */
-    Py_TPFLAGS_DEFAULT, /* tp_flags */
-    __Permset_Type_doc__,/* tp_doc */
-    0,                  /* tp_traverse */
-    0,                  /* tp_clear */
-    0,                  /* tp_richcompare */
-    0,                  /* tp_weaklistoffset */
-    0,                  /* tp_iter */
-    0,                  /* tp_iternext */
-    Permset_methods,    /* tp_methods */
-    0,                  /* tp_members */
-    Permset_getsets,    /* tp_getset */
-    0,                  /* tp_base */
-    0,                  /* tp_dict */
-    0,                  /* tp_descr_get */
-    0,                  /* tp_descr_set */
-    0,                  /* tp_dictoffset */
-    Permset_init,       /* tp_init */
-    0,                  /* tp_alloc */
-    Permset_new,        /* tp_new */
+    .tp_name = "posix1e.Permset",
+    .tp_basicsize = sizeof(Permset_Object),
+    .tp_itemsize = 0,
+    .tp_dealloc = Permset_dealloc,
+    .tp_str = Permset_str,
+    .tp_flags = Py_TPFLAGS_DEFAULT,
+    .tp_doc = __Permset_Type_doc__,
+    .tp_methods = Permset_methods,
+    .tp_getset = Permset_getsets,
+    .tp_init = Permset_init,
+    .tp_new = Permset_new,
 };
 
 #endif
@@ -1782,10 +1725,10 @@ static char __posix1e_doc__[] =
 
 static struct PyModuleDef posix1emodule = {
     PyModuleDef_HEAD_INIT,
-    "posix1e",
-    __posix1e_doc__,
-    0,
-    aclmodule_methods,
+    .m_name = "posix1e",
+    .m_doc = __posix1e_doc__,
+    .m_size = 0,
+    .m_methods = aclmodule_methods,
 };
 
 PyMODINIT_FUNC
