@@ -220,6 +220,19 @@ FD_D = [
     "file io stream",
 ]
 
+DIR_D = [
+    "directory",
+    "directory (bytes)",
+    "directory (path object)",
+]
+
+DIR_P = [
+    get_dir,
+    as_bytes(get_dir),
+    pytest.param(as_fspath(get_dir),
+                 marks=[NOT_BEFORE_36, NOT_PYPY]),
+]
+
 ALL_P = FILE_P + FD_P
 ALL_D = FILE_D + FD_D
 
@@ -233,6 +246,11 @@ def fd_subject(testdir, request):
     with request.param(testdir) as value:
         yield value
 
+@pytest.fixture(params=DIR_P, ids=DIR_D)
+def dir_subject(testdir, request):
+    with request.param(testdir) as value:
+        yield value
+
 @pytest.fixture(params=ALL_P, ids=ALL_D)
 def subject(testdir, request):
     with request.param(testdir) as value:
@@ -246,10 +264,9 @@ class TestLoad:
         acl = posix1e.ACL(file=file_subject)
         assert acl.valid()
 
-    def test_from_dir(self, testdir):
+    def test_from_dir(self, dir_subject):
         """Test loading ACLs from a directory"""
-        with get_dir(testdir) as dname:
-          acl2 = posix1e.ACL(filedef=dname)
+        acl2 = posix1e.ACL(filedef=dir_subject)
         # default ACLs might or might not be valid; missing ones are
         # not valid, so we don't test acl2 for validity
 
@@ -506,6 +523,39 @@ class TestAclExtensions:
         assert a == b
         assert b != c
 
+    @require_copy_ext
+    def test_acl_copy_ext_failure(self):
+        a = posix1e.ACL()
+        state = a.__getstate__()
+        # This is a dangerous test. The acl_copy_int() C function gets
+        # a void * buffer, and then casts that to an ACL structure,
+        # irrespective of buffer length; this can lead to segfaults
+        # (via unallocated memory indexing)
+        #
+        # To mitigate this, pass same buffer size as returned from the
+        # state, just nulled out - in the Linux version of the
+        # library, the first byte is the structure size and is tested
+        # for correct size, and a null byte will cause failure.
+        nulled = b'\x00' * len(state)
+        with pytest.raises(IOError):
+            a.__setstate__(nulled)
+
+    @require_copy_ext
+    def test_acl_copy_ext_failure(self):
+        a = posix1e.ACL(text=BASIC_ACL_TEXT)
+        b = posix1e.ACL()
+        c = posix1e.ACL(acl=a)
+        assert a == c
+        assert a != b
+        state = b.__getstate__()
+        # See notes in the test_acl_copy_ext_failure() for how tricky this is.
+        nulled = b'\x00' * len(state)
+        with pytest.raises(IOError):
+            a.__setstate__(nulled)
+        # Assert that 'a' didn't change in the attempt to restore
+        # invalid state.
+        assert a == c
+
     @require_copy_ext
     def test_acl_copy_ext_args(self):
         a = posix1e.ACL()
